<?php
/*
Class Name: VI_WOO_BOPO_BUNDLE_Admin
Author: Andy Ha (support@villatheme.com)
Author URI: http://villatheme.com
Copyright 2015 villatheme.com. All rights reserved.
*/
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class VI_WOO_BOPO_BUNDLE_Admin {
	protected $settings;
	protected $language;
	protected $languages;
	protected $default_language;
	protected $languages_data;
	protected $languages_count;

	function __construct() {
		register_activation_hook( __FILE__, array( $this, 'install' ) );
		$this->settings         = VI_WOO_BOPO_BUNDLE_DATA::get_instance();
		$this->languages        = array();
		$this->languages_count  = 0;
		$this->languages_data   = array();
		$this->default_language = '';
		add_filter(
			'plugin_action_links_bopo-woocommerce-product-bundle-builder/bopo-woocommerce-product-bundle-builder.php', array(
				$this,
				'settings_link'
			)
		);
		add_action( 'init', array( $this, 'init' ) );
		add_action( 'admin_menu', array( $this, 'add_menu' ) );
		add_action( 'admin_init', array( $this, 'save_settings' ), 99 );

		add_action( 'wp_print_scripts', array( $this, 'custom_script' ) );

		add_action( 'admin_enqueue_scripts', array( $this, 'admin_enqueue_script' ) );

	}

	function init() {
		bopobb_register_product_type();
		load_plugin_textdomain( 'bopo-woocommerce-product-bundle-builder' );
		$this->load_plugin_textDomain();
		$key = $this->settings->get_params( 'bopobb_update_key' );
		if ( class_exists( 'VillaTheme_Plugin_Check_Update' ) ) {
			$setting_url = admin_url( 'admin.php?page=bopo-woocommerce-product-bundle-builder' );
			new VillaTheme_Plugin_Check_Update (
				VI_WOO_BOPO_BUNDLE_VERSION,                    // current version
				'https://villatheme.com/wp-json/downloads/v3',  // update path
				'bopo-woocommerce-product-bundle-builder/bopo-woocommerce-product-bundle-builder.php',                  // plugin file slug
				'bopo-woocommerce-product-bundle-builder', '102285', $key, $setting_url
			);
			new VillaTheme_Plugin_Updater( 'bopo-woocommerce-product-bundle-builder/bopo-woocommerce-product-bundle-builder.php', 'bopo-woocommerce-product-bundle-builder', $setting_url );

		}
		if ( class_exists( 'VillaTheme_Support_Pro' ) ) {
			new VillaTheme_Support_Pro(
				array(
					'support'   => 'https://villatheme.com/supports/forum/plugins/bopo',
					'docs'      => 'https://docs.villatheme.com/?item=bopo',
					'review'    => 'https://codecanyon.net/downloads',
					'css'       => VI_WOO_BOPO_BUNDLE_CSS,
					'image'     => VI_WOO_BOPO_BUNDLE_IMAGES,
					'slug'      => 'bopo-woocommerce-product-bundle-builder',
					'menu_slug' => 'bopo-woocommerce-product-bundle-builder',
					'version'   => VI_WOO_BOPO_BUNDLE_VERSION
				)
			);
		}
	}

	/**
	 * When active plugin Function will be call
	 */
	public function install() {
		global $wp_version;
		If ( version_compare( $wp_version, "2.9", "<" ) ) {
			deactivate_plugins( basename( __FILE__ ) ); // Deactivate our plugin
			wp_die( "This plugin requires WordPress version 2.9 or higher." );
		}
	}

	function add_menu() {
		add_menu_page(
			esc_html__( 'Bopo - WooCommerce Product Bundle Builder Premium', 'bopo-woocommerce-product-bundle-builder' ),
			esc_html__( 'Bopo Bundle', 'bopo-woocommerce-product-bundle-builder' ),
			'manage_options',
			'bopo-woocommerce-product-bundle-builder',
			array(
				$this,
				'settings_page'
			), VI_WOO_BOPO_BUNDLE_IMAGES . 'icon_bopo.svg', 2
		);
		add_submenu_page(
			'bopo-woocommerce-product-bundle-builder',
			esc_html__( 'Bopo - WooCommerce Product Bundle Builder - Build Your Own Box', 'bopo-woocommerce-product-bundle-builder' ),
			esc_html__( 'Settings', 'bopo-woocommerce-product-bundle-builder' ),
			'manage_options',
			'bopo-woocommerce-product-bundle-builder'
		);
	}

	public function custom_script() {
		$script = 'var bopobb_ajax_url = "' . admin_url( 'admin-ajax.php' ) . '"'; ?>
        <script type="text/javascript" data-cfasync="false">
			<?php echo $script; ?>
        </script>
	<?php }

	function bopobb_wp_get_page_url_by_template_slug( $template_slug ) {
		$url      = null;
		$template = 'page-' . $template_slug . '.php';

		$pages = get_posts( array(
			'post_type'  => 'page',
			'meta_query' => array(
				array(
					'key'     => '_wp_page_template',
					'value'   => $template,
					'compare' => '=',
				)
			)
		) );

		if ( isset( $pages[0] ) ) {
			$url = get_permalink( $pages[0]->ID );
		}

		return $url;
	}

	/**
	 * Save data.
	 */
	public function save_settings() {
		global $bopobb_settings;
		if ( ! current_user_can( 'manage_options' ) ) {
			return;
		}
		/*wpml*/
		if ( is_plugin_active( 'sitepress-multilingual-cms/sitepress.php' ) ) {
			global $sitepress;
			$default_lang           = $sitepress->get_default_language();
			$this->default_language = $default_lang;
			$languages              = icl_get_languages( 'skip_missing=N&orderby=KEY&order=DIR&link_empty_to=str' );
			$this->languages_data   = $languages;
			if ( count( $languages ) ) {
				foreach ( $languages as $key => $language ) {
					if ( $key != $default_lang ) {
						$this->languages[] = $key;
					}
				}
			}
		} elseif ( class_exists( 'Polylang' ) ) {
			/*Polylang*/
			$languages    = pll_languages_list();
			$default_lang = pll_default_language( 'slug' );
			foreach ( $languages as $language ) {
				if ( $language == $default_lang ) {
					continue;
				}
				$this->languages[] = $language;
			}
		}
		$this->languages_count = count( $this->languages );

		if ( ! isset( $_POST['bopobb_nonce_field'] ) || ! wp_verify_nonce( $_POST['bopobb_nonce_field'], 'bopobb_settings_page_save' ) ) {
			return;
		}
		$args = array(
			//General
			'bopobb_bundle_quantity'   => 1,
			'bopobb_view_stock'        => 0,
			'bopobb_view_ratting'      => 0,
			'bopobb_view_description'  => 0,
			'bopobb_link_individual'   => 0,
			'bopobb_single_template'   => 1,
			'bopobb_item_column'       => 4,
			'bopobb_image_size'        => 'woocommerce_thumbnail',
			'bopobb_out_stock'         => 0,
			'bopobb_remove_ready_cart' => 0,
			'bopobb_edit_cart'         => 0,
			'bopobb_edit_checkout'     => 0,

			//swap button
			'bopobb_swap_text'         => 'Change',
			'bopobb_swap_pos'          => 1,
			'bopobb_swap_background'   => '',
			'bopobb_swap_color'        => '',

			//popup
			'bopobb_popup_title'       => 'Please select your product',
			'bopobb_popup_background'  => '',
			'bopobb_popup_color'       => '',
			'bopobb_popup_variations'  => 'list',
			'bopobb_popup_fontsize'    => '',
			'bopobb_popup_page_items'  => 32,

			'bopobb_custom_css'        => '',

			'bopobb_alert_products'    => 'Please select products for all items of the bundle.',
			'bopobb_alert_available'   => 'Please select product available.',
			'bopobb_alert_change'      => 'No product available to change.',
			'bopobb_alert_max'         => 'This item quantity must be less than',
			'bopobb_alert_min'         => 'This item quantity must be greater than',

			//update key
			'bopobb_update_key'        => isset( $_POST['bopobb_update_key'] ) ? sanitize_text_field( $_POST['bopobb_update_key'] ) : '',
		);
		if ( $this->languages_count ) {
			foreach ( $this->languages as $key => $value ) {
				$args[ 'bopobb_popup_title_' . $value ] = isset( $_POST[ 'bopobb_popup_title_' . $value ] ) ? sanitize_text_field( $_POST[ 'bopobb_popup_title_' . $value ] ) : 'Select product to bundle';
				$args[ 'bopobb_alert_products_' . $value ] = isset( $_POST[ 'bopobb_alert_products_' . $value ] ) ? sanitize_text_field( $_POST[ 'bopobb_alert_products_' . $value ] ) : 'Select product to bundle';
				$args[ 'bopobb_alert_available_' . $value ] = isset( $_POST[ 'bopobb_alert_available_' . $value ] ) ? sanitize_text_field( $_POST[ 'bopobb_alert_available_' . $value ] ) : 'Select product to bundle';
				$args[ 'bopobb_alert_change_' . $value ] = isset( $_POST[ 'bopobb_alert_change_' . $value ] ) ? sanitize_text_field( $_POST[ 'bopobb_alert_change_' . $value ] ) : 'Select product to bundle';
				$args[ 'bopobb_alert_max_' . $value ] = isset( $_POST[ 'bopobb_alert_max_' . $value ] ) ? sanitize_text_field( $_POST[ 'bopobb_alert_max_' . $value ] ) : 'Select product to bundle';
				$args[ 'bopobb_alert_min_' . $value ] = isset( $_POST[ 'bopobb_alert_min_' . $value ] ) ? sanitize_text_field( $_POST[ 'bopobb_alert_min_' . $value ] ) : 'Select product to bundle';
			}
		}
		foreach ( $args as $key => $arg ) {
			$args[ $key ] = isset( $_POST[ $key ] ) ? sanitize_text_field( $_POST[ $key ] ) : '';
		}
		if ( isset( $_POST['bopobb_save_data'] ) || isset( $_POST['bopobb_save_check'] ) ) {
			$args = wp_parse_args( $args, get_option( 'woo_bopo_bundle_params', $bopobb_settings ) );
			update_option( 'woo_bopo_bundle_params', $args );
			$bopobb_settings = $args;
			$this->settings  = VI_WOO_BOPO_BUNDLE_DATA::get_instance( true );

			if ( isset( $_POST['bopobb_save_check'] ) ) {
				delete_transient( '_site_transient_update_plugins' );
				delete_transient( 'villatheme_item_' );
				delete_option( 'bopo-woocommerce-product-bundle-builder_messages' );
			}

			add_action( 'admin_notices', function () {
				?>
                <div class="updated">
                    <p><?php esc_html_e( 'Your settings have been saved!', 'bopo-woocommerce-product-bundle-builder' ) ?></p>
                </div>
				<?php
			} );
		}
		if ( isset( $_POST['bopobb_save_check'] ) ) {
			delete_transient( '_site_transient_update_plugins' );
			delete_transient( 'villatheme_item_' );
			delete_option( 'bopo-woocommerce-product-bundle-builder_messages' );
		}
	}

	public function settings_page() {
		if ( ! current_user_can( 'manage_options' ) ) {
			return;
		}
		?>
        <div class="wrap">
            <h2><?php esc_html_e( 'Bopo - WooCommerce Product Bundle Builder - Build Your Own Box', 'bopo-woocommerce-product-bundle-builder' ); ?></h2>
            <div class="vi-ui raised">
                <form class="vi-ui form bopobb-general-settings" method="post" action="">
					<?php
					wp_nonce_field( 'bopobb_settings_page_save', 'bopobb_nonce_field' );
					?>
                    <div class="vi-ui top attached tabular menu">
                        <div class="item active"
                             data-tab="general"><?php esc_html_e( 'General', 'bopo-woocommerce-product-bundle-builder' ); ?></div>
                        <div class="item"
                             data-tab="layout"><?php esc_html_e( 'Layout', 'bopo-woocommerce-product-bundle-builder' ); ?></div>
                        <div class="item"
                             data-tab="update"><?php esc_html_e( 'Update', 'bopo-woocommerce-product-bundle-builder' ); ?></div>
                    </div>
                    <div class="vi-ui bottom attached active tab segment" data-tab="general">
                        <table class="vi-ui bottom attached form-table">
                            <tbody>
                            <tr valign="top">
                                <th scope="row">
                                    <label for="bopobb_out_stock">
										<?php esc_html_e( 'Show out of stock product', 'bopo-woocommerce-product-bundle-builder' ) ?></label>
                                </th>
                                <td>
                                    <div class="vi-ui toggle checkbox">
                                        <input type="checkbox" tabindex="6"
                                               id="bopobb_out_stock" <?php checked( $this->settings->get_params( 'bopobb_out_stock' ), 1 ) ?>
                                               name="bopobb_out_stock" value="1">
                                        <label></label>
                                    </div>
                                    <p class="description"><?php esc_html_e( '', 'bopo-woocommerce-product-bundle-builder' ); ?></p>
                                </td>
                            </tr>
                            <tr valign="top">
                                <th scope="row">
                                    <label for="bopobb_remove_ready_cart">
										<?php esc_html_e( 'Remove duplicate product outside bundle', 'bopo-woocommerce-product-bundle-builder' ) ?></label>
                                </th>
                                <td>
                                    <div class="vi-ui toggle checkbox">
                                        <input type="checkbox" tabindex="6"
                                               id="bopobb_remove_ready_cart" <?php checked( $this->settings->get_params( 'bopobb_remove_ready_cart' ), 1 ) ?>
                                               name="bopobb_remove_ready_cart" value="1">
                                        <label></label>
                                    </div>
                                    <p class="description"><?php esc_html_e( '', 'bopo-woocommerce-product-bundle-builder' ); ?></p>
                                </td>
                            </tr>
                            <tr valign="top">
                                <th scope="row">
                                    <label for="bopobb_edit_cart">
										<?php esc_html_e( 'Edit bundle item on cart', 'bopo-woocommerce-product-bundle-builder' ) ?></label>
                                </th>
                                <td>
                                    <div class="vi-ui toggle checkbox">
                                        <input type="checkbox" tabindex="6"
                                               id="bopobb_edit_cart" <?php checked( $this->settings->get_params( 'bopobb_edit_cart' ), 1 ) ?>
                                               name="bopobb_edit_cart" value="1">
                                        <label></label>
                                    </div>
                                    <p class="description"><?php esc_html_e( '', 'bopo-woocommerce-product-bundle-builder' ); ?></p>
                                </td>
                            </tr>
                            <tr valign="top">
                                <th scope="row">
                                    <label for="bopobb_edit_checkout">
										<?php esc_html_e( 'Edit bundle item on checkout', 'bopo-woocommerce-product-bundle-builder' ) ?></label>
                                </th>
                                <td>
                                    <div class="vi-ui toggle checkbox">
                                        <input type="checkbox" tabindex="6"
                                               id="bopobb_edit_checkout" <?php checked( $this->settings->get_params( 'bopobb_edit_checkout' ), 1 ) ?>
                                               name="bopobb_edit_checkout" value="1">
                                        <label></label>
                                    </div>
                                    <p class="description"><?php esc_html_e( '', 'bopo-woocommerce-product-bundle-builder' ); ?></p>
                                </td>
                            </tr>

                            </tbody>
                        </table>
                    </div>
                    <div class="vi-ui bottom attached tab segment" data-tab="layout">
                        <table class="vi-ui bottom attached form-table">
                            <tbody>
                            <tr valign="top">
                                <th scope="row">
                                    <h4 class="vi-ui blue header"><?php esc_html_e( 'Bundle view', 'bopo-woocommerce-product-bundle-builder' ); ?></h4>
                                </th>
                            </tr>
                            <tr valign="top">
                                <th scope="row">
                                    <label for="bopobb_bundle_quantity">
			                            <?php esc_html_e( 'Bundle show quantity', 'bopo-woocommerce-product-bundle-builder' ); ?>
                                    </label>
                                </th>
                                <td>
                                    <div class="vi-ui toggle checkbox">
                                        <input type="checkbox" tabindex="1"
                                               id="bopobb_bundle_quantity" <?php checked( $this->settings->get_params( 'bopobb_bundle_quantity' ), 1 ) ?>
                                               name="bopobb_bundle_quantity" value="1">
                                        <label></label>
                                    </div>
                                </td>
                            </tr>
                            <tr valign="top">
                                <th scope="row">
                                    <label for="bopobb_view_stock">
										<?php esc_html_e( 'Product detail', 'bopo-woocommerce-product-bundle-builder' ); ?>
                                    </label>
                                </th>
                                <td>
                                    <div class="vi-ui toggle checkbox">
                                        <input type="checkbox" tabindex="1"
                                               id="bopobb_view_stock" <?php checked( $this->settings->get_params( 'bopobb_view_stock' ), 1 ) ?>
                                               name="bopobb_view_stock" value="1">
                                        <label><?php esc_html_e( 'Show stock', 'bopo-woocommerce-product-bundle-builder' ); ?></label>
                                    </div>
                                </td>
                            </tr>
                            <tr valign="top">
                                <th scope="row">
                                    <label for="bopobb_view_ratting">
                                    </label>
                                </th>
                                <td>
                                    <div class="vi-ui toggle checkbox">
                                        <input type="checkbox" tabindex="2"
                                               id="bopobb_view_ratting" <?php checked( $this->settings->get_params( 'bopobb_view_ratting' ), 1 ) ?>
                                               name="bopobb_view_ratting" value="1">
                                        <label><?php esc_html_e( 'Show ratting', 'bopo-woocommerce-product-bundle-builder' ); ?></label>
                                    </div>
                                </td>
                            </tr>
                            <tr valign="top">
                                <th scope="row">
                                    <label for="bopobb_view_description">
                                    </label>
                                </th>
                                <td>
                                    <div class="vi-ui toggle checkbox">
                                        <input type="checkbox" tabindex="3"
                                               id="bopobb_view_description" <?php checked( $this->settings->get_params( 'bopobb_view_description' ), 1 ) ?>
                                               name="bopobb_view_description" value="1">
                                        <label><?php esc_html_e( 'Show short description(on hover)', 'bopo-woocommerce-product-bundle-builder' ); ?></label>
                                    </div>
                                </td>
                            </tr>
                            <tr valign="top">
                                <th scope="row">
                                    <label for="bopobb_link_individual">
										<?php esc_html_e( 'Link to individual product', 'bopo-woocommerce-product-bundle-builder' ) ?></label>
                                </th>
                                <td>
                                    <select class="vi-ui fluid dropdown" id="bopobb_link_individual" tabindex="4"
                                            name="bopobb_link_individual">
                                        <option <?php selected( $this->settings->get_params( 'bopobb_link_individual' ), 0 ) ?>
                                                value="0"><?php esc_html_e( 'No link', 'bopo-woocommerce-product-bundle-builder' ) ?></option>
                                        <option <?php selected( $this->settings->get_params( 'bopobb_link_individual' ), 1 ) ?>
                                                value="1"><?php esc_html_e( 'New tab', 'bopo-woocommerce-product-bundle-builder' ) ?></option>
                                        <option <?php selected( $this->settings->get_params( 'bopobb_link_individual' ), 2 ) ?>
                                                value="2"><?php esc_html_e( 'Bundle change product popup', 'bopo-woocommerce-product-bundle-builder' ) ?></option>
                                    </select>
                                    <p class="description"><?php esc_html_e( 'Action after click on title product of bundle', 'bopo-woocommerce-product-bundle-builder' ); ?></p>
                                </td>
                            </tr>
                            <tr valign="top">
                                <th scope="row">
                                    <label for="bopobb_single_template">
										<?php esc_html_e( 'Template', 'bopo-woocommerce-product-bundle-builder' ) ?></label>
                                </th>
                                <td>
                                    <select class="vi-ui fluid dropdown" id="bopobb_single_template" tabindex="5"
                                            name="bopobb_single_template">
                                        <option <?php selected( $this->settings->get_params( 'bopobb_single_template' ), 1 ) ?>
                                                value="1"><?php esc_html_e( 'Vertical bundle template', 'bopo-woocommerce-product-bundle-builder' ) ?></option>
                                        <option <?php selected( $this->settings->get_params( 'bopobb_single_template' ), 2 ) ?>
                                                value="2"><?php esc_html_e( 'Horizontal bundle template', 'bopo-woocommerce-product-bundle-builder' ) ?></option>
                                    </select>
                                    <p class="description"><?php esc_html_e( '', 'bopo-woocommerce-product-bundle-builder' ); ?></p>
                                </td>
                            </tr>
                            <tr valign="top">
                                <th scope="row">
                                    <label for="bopobb_item_column">
										<?php esc_html_e( 'Item column', 'bopo-woocommerce-product-bundle-builder' ) ?></label>
                                </th>
                                <td>
                                    <select class="vi-ui fluid dropdown" id="bopobb_item_column" tabindex="5"
                                            name="bopobb_item_column">
                                        <option <?php selected( $this->settings->get_params( 'bopobb_item_column' ), 2 ) ?>
                                                value="2">2
                                        </option>
                                        <option <?php selected( $this->settings->get_params( 'bopobb_item_column' ), 3 ) ?>
                                                value="3">3
                                        </option>
                                        <option <?php selected( $this->settings->get_params( 'bopobb_item_column' ), 4 ) ?>
                                                value="4">4
                                        </option>
                                    </select>
                                    <p class="description"><?php esc_html_e( '', 'bopo-woocommerce-product-bundle-builder' ); ?></p>
                                </td>
                            </tr>
                            <tr valign="top">
                                <th scope="row">
                                    <label for="bopobb_image_size">
										<?php esc_html_e( 'Image size', 'bopo-woocommerce-product-bundle-builder' ) ?></label>
                                </th>
                                <td>
									<?php
									global $_wp_additional_image_sizes;
									?>
                                    <select class="vi-ui fluid dropdown" id="bopobb_image_size" tabindex="5"
                                            name="bopobb_image_size">
										<?php
										$size_options = wp_get_additional_image_sizes();
										foreach ( $size_options as $key => $args ) {
											$height = $args['height'] ? $args['height'] : 'auto';
											$width  = $args['width'] ? $args['width'] : 'auto';
											$r      = "{$key} ({$width}x{$height})";
											?>
                                            <option <?php selected( $this->settings->get_params( 'bopobb_image_size' ), $key ) ?>
                                                    value="<?php echo esc_attr( $key ) ?>"><?php echo esc_html( $r ) ?></option>
											<?php
										}
										?>
                                    </select>
                                    <p class="description"><?php esc_html_e( '', 'bopo-woocommerce-product-bundle-builder' ); ?></p>
                                </td>
                            </tr>
                            <tr valign="top">
                                <th scope="row">
                                    <h4 class="vi-ui blue header"><?php esc_html_e( 'Popup change product', 'bopo-woocommerce-product-bundle-builder' ); ?></h4>
                                </th>
                            </tr>
                            <tr valign="top">
                                <th scope="row">
                                    <label for="bopobb_popup_title">
										<?php esc_html_e( 'Title', 'bopo-woocommerce-product-bundle-builder' ) ?></label>
                                </th>
                                <td>
                                    <input type="text" id="bopobb_popup_title" tabindex="10" name="bopobb_popup_title"
                                           value="<?php echo esc_attr( $this->settings->get_params( 'bopobb_popup_title' ) ); ?>"
                                           placeholder="<?php esc_attr_e( 'Select product to bundle', 'bopo-woocommerce-product-bundle-builder' ) ?>">
									<?php
									if ( $this->languages_count ) {
										foreach ( $this->languages as $key => $value ) {
											?>
                                            <p>
                                                <label for="<?php echo 'bopobb_popup_title_' . $value; ?>"><?php
													if ( isset( $this->languages_data[ $value ]['country_flag_url'] ) && $this->languages_data[ $value ]['country_flag_url'] ) {
														?>
                                                        <img src="<?php echo esc_url( $this->languages_data[ $value ]['country_flag_url'] ); ?>">
														<?php
													}
													echo $value;
													if ( isset( $this->languages_data[ $value ]['translated_name'] ) ) {
														echo '(' . $this->languages_data[ $value ]['translated_name'] . ')';
													}
													?>:</label>
                                            </p>
                                            <input type="text"
                                                   id="<?php echo esc_attr( 'bopobb_popup_title_' . $value ) ?>"
                                                   tabindex="7"
                                                   name="<?php echo esc_attr( 'bopobb_popup_title_' . $value ) ?>"
                                                   value="<?php echo esc_attr( $this->settings->get_params( 'bopobb_popup_title_' . $value ) ); ?>"
                                                   placeholder="<?php esc_attr_e( 'Please select your product', 'bopo-woocommerce-product-bundle-builder' ) ?>">
											<?php
										}
									}
									?>
                                </td>
                            </tr>
                            <tr valign="top">
                                <th scope="row">
                                    <label for="bopobb_popup_color">
										<?php esc_html_e( 'Color', 'bopo-woocommerce-product-bundle-builder' ) ?></label>
                                </th>
                                <td>
                                    <input name="bopobb_popup_color" id="bopobb_popup_color" tabindex="11" type="text"
                                           class="color-picker"
                                           value="<?php if ( $this->settings->get_params( 'bopobb_popup_color' ) ) {
										       echo esc_attr( $this->settings->get_params( 'bopobb_popup_color' ) );
									       } ?>"
                                           style="background: <?php if ( $this->settings->get_params( 'bopobb_popup_color' ) ) {
										       echo esc_attr( $this->settings->get_params( 'bopobb_popup_color' ) );
									       } ?>;"/>
                                </td>
                            </tr>
                            <tr valign="top">
                                <th scope="row">
                                    <label for="bopobb_popup_background">
										<?php esc_html_e( 'Background color', 'bopo-woocommerce-product-bundle-builder' ) ?></label>
                                </th>
                                <td>
                                    <input name="bopobb_popup_background" id="bopobb_popup_background" tabindex="12"
                                           type="text" class="color-picker"
                                           value="<?php if ( $this->settings->get_params( 'bopobb_popup_background' ) ) {
										       echo esc_attr( $this->settings->get_params( 'bopobb_popup_background' ) );
									       } ?>"
                                           style="background: <?php if ( $this->settings->get_params( 'bopobb_popup_background' ) ) {
										       echo esc_attr( $this->settings->get_params( 'bopobb_popup_background' ) );
									       } ?>;"/>
                                </td>
                            </tr>
                            <tr valign="top">
                                <th scope="row">
                                    <label for="bopobb_popup_page_items">

										<?php esc_html_e( 'Number product per page', 'bopo-woocommerce-product-bundle-builder' ) ?></label>
                                </th>
                                <td>
                                    <input type="number" name="bopobb_popup_page_items" id="bopobb_popup_page_items"
                                           tabindex="13" type="text" class="bopobb-popup-column"
                                           step="1" min="1" max="100"
                                           value="<?php if ( $this->settings->get_params( 'bopobb_popup_page_items' ) ) {
										       echo esc_attr( $this->settings->get_params( 'bopobb_popup_page_items' ) );
									       } ?>"/>
                                </td>
                            </tr>
                            <tr valign="top">
                                <th scope="row">
                                    <label for="bopobb_popup_variations">

										<?php esc_html_e( 'Select variations type', 'bopo-woocommerce-product-bundle-builder' ) ?></label>
                                </th>
                                <td>
                                    <select class="vi-ui fluid dropdown" id="bopobb_popup_variations" tabindex="5"
                                            name="bopobb_popup_variations">
                                        <option <?php selected( $this->settings->get_params( 'bopobb_popup_variations' ), 'list' ) ?>
                                                value="list">List of variations
                                        </option>
                                        <option <?php selected( $this->settings->get_params( 'bopobb_popup_variations' ), 'select' ) ?>
                                                value="select">Select box
                                        </option>
                                    </select>
                                    <p class="description"><?php esc_html_e( '', 'bopo-woocommerce-product-bundle-builder' ); ?></p>
                                </td>
                            </tr>
                            <tr valign="top">
                                <th scope="row">
                                    <h4 class="vi-ui blue header"><?php esc_html_e( 'Alert Title', 'bopo-woocommerce-product-bundle-builder' ); ?></h4>
                                </th>
                            </tr>
                            <tr valign="top">
                                <th scope="row">
                                    <label for="bopobb_alert_products">
			                            <?php esc_html_e( 'Products require', 'bopo-woocommerce-product-bundle-builder' ) ?></label>
                                </th>
                                <td>
                                    <input type="text" name="bopobb_alert_products" id="bopobb_alert_products"
                                           tabindex="13" type="text" class="bopobb-popup-column"
                                           value="<?php echo esc_attr( $this->settings->get_params( 'bopobb_alert_products' ) ) ?>"
                                           placeholder="<?php esc_attr_e( 'Please select products for all items of the bundle.', 'bopo-woocommerce-product-bundle-builder' ) ?>"/>
	                                <?php
	                                if ( $this->languages_count ) {
		                                foreach ( $this->languages as $key => $value ) {
			                                ?>
                                            <p>
                                                <label for="<?php echo 'bopobb_alert_products_' . $value; ?>"><?php
					                                if ( isset( $this->languages_data[ $value ]['country_flag_url'] ) && $this->languages_data[ $value ]['country_flag_url'] ) {
						                                ?>
                                                        <img src="<?php echo esc_url( $this->languages_data[ $value ]['country_flag_url'] ); ?>">
						                                <?php
					                                }
					                                echo $value;
					                                if ( isset( $this->languages_data[ $value ]['translated_name'] ) ) {
						                                echo '(' . $this->languages_data[ $value ]['translated_name'] . ')';
					                                }
					                                ?>:</label>
                                            </p>
                                            <input type="text" name="<?php echo esc_attr( 'bopobb_alert_products_' . $value ) ?>" id="<?php echo esc_attr( 'bopobb_alert_products_' . $value ) ?>"
                                                   tabindex="13" type="text" class="bopobb-popup-column"
                                                   value="<?php echo esc_attr( $this->settings->get_params( 'bopobb_alert_products_' . $value ) ); ?>"
                                                   placeholder="<?php esc_attr_e( 'Please select products for all items of the bundle.', 'bopo-woocommerce-product-bundle-builder' ) ?>"/>
			                                <?php
		                                }
	                                }
	                                ?>
                                </td>
                            </tr>
                            <tr valign="top">
                                <th scope="row">
                                    <label for="bopobb_alert_available">
			                            <?php esc_html_e( 'Purchasable require', 'bopo-woocommerce-product-bundle-builder' ) ?></label>
                                </th>
                                <td>
                                    <input type="text" name="bopobb_alert_available" id="bopobb_alert_available"
                                           tabindex="13" type="text" class="bopobb-popup-column"
                                           value="<?php echo esc_attr( $this->settings->get_params( 'bopobb_alert_available' ) ) ?>"
                                           placeholder="<?php esc_attr_e( 'Please select product available.', 'bopo-woocommerce-product-bundle-builder' ) ?>"/>
	                                <?php
	                                if ( $this->languages_count ) {
		                                foreach ( $this->languages as $key => $value ) {
			                                ?>
                                            <p>
                                                <label for="<?php echo 'bopobb_alert_available_' . $value; ?>"><?php
					                                if ( isset( $this->languages_data[ $value ]['country_flag_url'] ) && $this->languages_data[ $value ]['country_flag_url'] ) {
						                                ?>
                                                        <img src="<?php echo esc_url( $this->languages_data[ $value ]['country_flag_url'] ); ?>">
						                                <?php
					                                }
					                                echo $value;
					                                if ( isset( $this->languages_data[ $value ]['translated_name'] ) ) {
						                                echo '(' . $this->languages_data[ $value ]['translated_name'] . ')';
					                                }
					                                ?>:</label>
                                            </p>
                                            <input type="text" name="<?php echo esc_attr( 'bopobb_alert_available_' . $value ) ?>" id="<?php echo esc_attr( 'bopobb_alert_available_' . $value ) ?>"
                                                   tabindex="13" type="text" class="bopobb-popup-column"
                                                   value="<?php echo esc_attr( $this->settings->get_params( 'bopobb_alert_available_' . $value ) ); ?>"
                                                   placeholder="<?php esc_attr_e( 'Please select product available.', 'bopo-woocommerce-product-bundle-builder' ) ?>"/>
			                                <?php
		                                }
	                                }
	                                ?>
                                </td>
                            </tr>
                            <tr valign="top">
                                <th scope="row">
                                    <label for="bopobb_alert_change">
			                            <?php esc_html_e( "Can't change item", "bopo-woocommerce-product-bundle-builder" ) ?></label>
                                </th>
                                <td>
                                    <input type="text" name="bopobb_alert_change" id="bopobb_alert_change"
                                           tabindex="13" type="text" class="bopobb-popup-column"
                                           value="<?php echo esc_attr( $this->settings->get_params( 'bopobb_alert_change' ) ) ?>"
                                           placeholder="<?php esc_attr_e( 'No product available to change.', 'bopo-woocommerce-product-bundle-builder' ) ?>"/>
	                                <?php
	                                if ( $this->languages_count ) {
		                                foreach ( $this->languages as $key => $value ) {
			                                ?>
                                            <p>
                                                <label for="<?php echo 'bopobb_alert_change_' . $value; ?>"><?php
					                                if ( isset( $this->languages_data[ $value ]['country_flag_url'] ) && $this->languages_data[ $value ]['country_flag_url'] ) {
						                                ?>
                                                        <img src="<?php echo esc_url( $this->languages_data[ $value ]['country_flag_url'] ); ?>">
						                                <?php
					                                }
					                                echo $value;
					                                if ( isset( $this->languages_data[ $value ]['translated_name'] ) ) {
						                                echo '(' . $this->languages_data[ $value ]['translated_name'] . ')';
					                                }
					                                ?>:</label>
                                            </p>
                                            <input type="text" name="<?php echo esc_attr( 'bopobb_alert_change_' . $value ) ?>" id="<?php echo esc_attr( 'bopobb_alert_change_' . $value ) ?>"
                                                   tabindex="13" type="text" class="bopobb-popup-column"
                                                   value="<?php echo esc_attr( $this->settings->get_params( 'bopobb_alert_change_' . $value ) ); ?>"
                                                   placeholder="<?php esc_attr_e( 'No product available to change.', 'bopo-woocommerce-product-bundle-builder' ) ?>"/>
			                                <?php
		                                }
	                                }
	                                ?>
                                </td>
                            </tr>
                            <tr valign="top">
                                <th scope="row">
                                    <label for="bopobb_alert_max">
			                            <?php esc_html_e( 'Quantity max require', 'bopo-woocommerce-product-bundle-builder' ) ?></label>
                                </th>
                                <td>
                                    <input type="text" name="bopobb_alert_max" id="bopobb_alert_max"
                                           tabindex="13" type="text" class="bopobb-popup-column"
                                           value="<?php echo esc_attr( $this->settings->get_params( 'bopobb_alert_max' ) ) ?>"
                                           placeholder="<?php esc_attr_e( 'This item quantity must be less than', 'bopo-woocommerce-product-bundle-builder' ) ?>"/>
	                                <?php
	                                if ( $this->languages_count ) {
		                                foreach ( $this->languages as $key => $value ) {
			                                ?>
                                            <p>
                                                <label for="<?php echo 'bopobb_alert_max_' . $value; ?>"><?php
					                                if ( isset( $this->languages_data[ $value ]['country_flag_url'] ) && $this->languages_data[ $value ]['country_flag_url'] ) {
						                                ?>
                                                        <img src="<?php echo esc_url( $this->languages_data[ $value ]['country_flag_url'] ); ?>">
						                                <?php
					                                }
					                                echo $value;
					                                if ( isset( $this->languages_data[ $value ]['translated_name'] ) ) {
						                                echo '(' . $this->languages_data[ $value ]['translated_name'] . ')';
					                                }
					                                ?>:</label>
                                            </p>
                                            <input type="text" name="<?php echo esc_attr( 'bopobb_alert_max_' . $value ) ?>" id="<?php echo esc_attr( 'bopobb_alert_max_' . $value ) ?>"
                                                   tabindex="13" type="text" class="bopobb-popup-column"
                                                   value="<?php echo esc_attr( $this->settings->get_params( 'bopobb_alert_max_' . $value ) ); ?>"
                                                   placeholder="<?php esc_attr_e( 'This item quantity must be less than', 'bopo-woocommerce-product-bundle-builder' ) ?>"/>
			                                <?php
		                                }
	                                }
	                                ?>
                                </td>
                            </tr>
                            <tr valign="top">
                                <th scope="row">
                                    <label for="bopobb_alert_min">
			                            <?php esc_html_e( 'Quantity min require', 'bopo-woocommerce-product-bundle-builder' ) ?></label>
                                </th>
                                <td>
                                    <input type="text" name="bopobb_alert_min" id="bopobb_alert_min"
                                           tabindex="13" type="text" class="bopobb-popup-column"
                                           value="<?php echo esc_attr( $this->settings->get_params( 'bopobb_alert_min' ) ) ?>"
                                           placeholder="<?php esc_attr_e( 'This item quantity must be greater than', 'bopo-woocommerce-product-bundle-builder' ) ?>"/>
	                                <?php
	                                if ( $this->languages_count ) {
		                                foreach ( $this->languages as $key => $value ) {
			                                ?>
                                            <p>
                                                <label for="<?php echo 'bopobb_alert_min_' . $value; ?>"><?php
					                                if ( isset( $this->languages_data[ $value ]['country_flag_url'] ) && $this->languages_data[ $value ]['country_flag_url'] ) {
						                                ?>
                                                        <img src="<?php echo esc_url( $this->languages_data[ $value ]['country_flag_url'] ); ?>">
						                                <?php
					                                }
					                                echo $value;
					                                if ( isset( $this->languages_data[ $value ]['translated_name'] ) ) {
						                                echo '(' . $this->languages_data[ $value ]['translated_name'] . ')';
					                                }
					                                ?>:</label>
                                            </p>
                                            <input type="text" name="<?php echo esc_attr( 'bopobb_alert_min_' . $value ) ?>" id="<?php echo esc_attr( 'bopobb_alert_min_' . $value ) ?>"
                                                   tabindex="13" type="text" class="bopobb-popup-column"
                                                   value="<?php echo esc_attr( $this->settings->get_params( 'bopobb_alert_min_' . $value ) ); ?>"
                                                   placeholder="<?php esc_attr_e( 'This item quantity must be greater than', 'bopo-woocommerce-product-bundle-builder' ) ?>"/>
			                                <?php
		                                }
	                                }
	                                ?>
                                </td>
                            </tr>
                            <tr valign="top">
                                <th scope="row">
                                    <h4 class="vi-ui blue header"><?php esc_html_e( 'Custom Script', 'bopo-woocommerce-product-bundle-builder' ); ?></h4>
                                </th>
                            </tr>
                            <tr valign="top">
                                <th scope="row">
                                    <label for="bopobb_custom_css">
			                            <?php esc_html_e( 'Custom CSS', 'bopo-woocommerce-product-bundle-builder' ) ?></label>
                                </th>
                                <td>
                                    <textarea type="text" name="bopobb_custom_css" id="bopobb_custom_css"
                                           tabindex="14" class="bopobb-popup-column"/><?php esc_html( $this->settings->get_params( 'bopobb_custom_css' ) ); ?>
                                    </textarea>
                                </td>
                            </tr>
                            </tbody>
                        </table>
                    </div>
                    <div class="vi-ui bottom attached tab segment" data-tab="update">
                        <table class="form-table">
                            <tbody>
                            <tr valign="top">
                                <th scope="row">
                                    <label for="auto-update-key"></label><?php esc_html_e( 'Auto update key', 'bopo-woocommerce-product-bundle-builder' ) ?>
                                </th>
                                <td>
                                    <div class="fields">
                                        <div class="ten wide field">
                                            <input type="text" name="bopobb_update_key" id="auto-update-key"
                                                   class="villatheme-autoupdate-key-field"
                                                   value="<?php echo $this->settings->get_params( 'bopobb_update_key' ); ?>">
                                        </div>
                                        <div class="six wide field">
                                        <span class="vi-ui button green villatheme-get-key-button"
                                              data-href="https://api.envato.com/authorization?response_type=code&client_id=villatheme-download-keys-6wzzaeue&redirect_uri=https://villatheme.com/update-key"
                                              data-id="35677459"><?php echo esc_html__( 'Get Key', 'bopo-woocommerce-product-bundle-builder' ) ?></span>
                                        </div>
                                    </div>
									<?php do_action( 'bopo-woocommerce-product-bundle-builder_key' ) ?>
                                    <p class="description"><?php _e( 'Please fill your key what you get from <a target="_blank" href="https://villatheme.com/my-download">https://villatheme.com/my-download</a>. See <a target="_blank" href="https://villatheme.com/knowledge-base/how-to-use-auto-update-feature/">guide</a>.', 'bopo-woocommerce-product-bundle-builder' ) ?>
                                    </p>
                                </td>
                            </tr>
                            </tbody>
                        </table>
                    </div>
                    <p>
                        <button class="vi-ui primary button labeled icon bopobb-submit" name="bopobb_save_data"><i
                                    class="icon save"></i><?php esc_html_e( 'Save', 'bopo-woocommerce-product-bundle-builder' ); ?>
                        </button>
                        <button type="submit" class="vi-ui button labeled icon"
                                name="bopobb_save_check"><i
                                    class="icon save"></i><?php esc_html_e( 'Save & Check Key', 'bopo-woocommerce-product-bundle-builder' ); ?>
                        </button>
                    </p>
                </form>
            </div>
        </div>
		<?php
		do_action( 'villatheme_support_bopo-woocommerce-product-bundle-builder' );
	}

	/**
	 * Link to Settings
	 *
	 * @param $links
	 *
	 * @return mixed
	 */
	public function settings_link( $links ) {
		$settings_link = '<a href="admin.php?page=bopo-woocommerce-product-bundle-builder" title="' . __( 'Bopo - WooCommerce Product Bundle Builder - Build Your Own Box', 'bopo-woocommerce-product-bundle-builder' ) . '">' . __( 'Settings', 'bopo-woocommerce-product-bundle-builder' ) . '</a>';
		array_unshift( $links, $settings_link );

		return $links;
	}

	/**
	 * load Language translate
	 */
	function load_plugin_textDomain() {
		$locale = apply_filters( 'plugin_locale', get_locale(), 'bopo-woocommerce-product-bundle-builder' );
		// Admin Locale
		if ( is_admin() ) {
			load_textdomain( 'bopo-woocommerce-product-bundle-builder', VI_WOO_BOPO_BUNDLE_LANGUAGES . "bopo-woocommerce-product-bundle-builder-$locale.mo" );
		}

		// Global + Frontend Locale
		load_textdomain( 'bopo-woocommerce-product-bundle-builder', VI_WOO_BOPO_BUNDLE_LANGUAGES . "bopo-woocommerce-product-bundle-builder-$locale.mo" );
		load_plugin_textdomain( 'bopo-woocommerce-product-bundle-builder', false, VI_WOO_BOPO_BUNDLE_LANGUAGES );
	}

	/**
	 *  Include style and script
	 */
	public function admin_enqueue_script() {
		$page = isset( $_REQUEST['page'] ) ? wp_unslash( sanitize_text_field( $_REQUEST['page'] ) ) : '';
		if ( $page == 'bopo-woocommerce-product-bundle-builder' ) {
			// style
			wp_enqueue_style( 'woo-bopo-bundle-semantic-button-css', VI_WOO_BOPO_BUNDLE_CSS . 'button.min.css', array(), VI_WOO_BOPO_BUNDLE_VERSION );
			wp_enqueue_style( 'woo-bopo-bundle-semantic-table-css', VI_WOO_BOPO_BUNDLE_CSS . 'table.min.css', array(), VI_WOO_BOPO_BUNDLE_VERSION );
			wp_enqueue_style( 'woo-bopo-bundle-semantic-form-css', VI_WOO_BOPO_BUNDLE_CSS . 'form.min.css', array(), VI_WOO_BOPO_BUNDLE_VERSION );
			wp_enqueue_style( 'woo-bopo-bundle-semantic-dropdown-css', VI_WOO_BOPO_BUNDLE_CSS . 'dropdown.min.css', array(), VI_WOO_BOPO_BUNDLE_VERSION );
			wp_enqueue_style( 'woo-bopo-bundle-semantic-transition-css', VI_WOO_BOPO_BUNDLE_CSS . 'transition.min.css', VI_WOO_BOPO_BUNDLE_VERSION );
			wp_enqueue_style( 'woo-bopo-bundle-semantic-checkbox-css', VI_WOO_BOPO_BUNDLE_CSS . 'checkbox.min.css', array(), VI_WOO_BOPO_BUNDLE_VERSION );
			wp_enqueue_style( 'woo-bopo-bundle-semantic-menu-css', VI_WOO_BOPO_BUNDLE_CSS . 'menu.min.css', array(), VI_WOO_BOPO_BUNDLE_VERSION );
			wp_enqueue_style( 'woo-bopo-bundle-semantic-header-css', VI_WOO_BOPO_BUNDLE_CSS . 'header.min.css', array(), VI_WOO_BOPO_BUNDLE_VERSION );
			wp_enqueue_style( 'woo-bopo-bundle-semantic-segment-css', VI_WOO_BOPO_BUNDLE_CSS . 'segment.min.css', array(), VI_WOO_BOPO_BUNDLE_VERSION );
			wp_enqueue_style( 'woo-bopo-bundle-semantic-icon-css', VI_WOO_BOPO_BUNDLE_CSS . 'icon.min.css', array(), VI_WOO_BOPO_BUNDLE_VERSION );
			wp_enqueue_style( 'woo-bopo-bundle-semantic-tab-css', VI_WOO_BOPO_BUNDLE_CSS . 'tab.min.css', array(), VI_WOO_BOPO_BUNDLE_VERSION );

			//script
			wp_enqueue_script(
				'iris', admin_url( 'js/iris.min.js' ), array(
				'jquery-ui-draggable',
				'jquery-ui-slider',
				'jquery-touch-punch'
			), false, 1
			);
			if ( WP_DEBUG ) {
				wp_enqueue_style( 'woo-bopo-bundle-setting-css', VI_WOO_BOPO_BUNDLE_CSS . 'bopo-settings.css' );
				wp_enqueue_script( 'woo-bopo-bundle-setting', VI_WOO_BOPO_BUNDLE_JS . 'bopo-settings.js', array( 'jquery' ), VI_WOO_BOPO_BUNDLE_VERSION );
			} else {
				wp_enqueue_style( 'woo-bopo-bundle-setting-css', VI_WOO_BOPO_BUNDLE_CSS . 'bopo-settings.min.css' );
				wp_enqueue_script( 'woo-bopo-bundle-setting', VI_WOO_BOPO_BUNDLE_JS . 'bopo-settings.min.js', array( 'jquery' ), VI_WOO_BOPO_BUNDLE_VERSION );
			}
			wp_enqueue_script( 'woo-bopo-bundle-semantic-tab-js', VI_WOO_BOPO_BUNDLE_JS . 'tab.js', array( 'jquery' ), VI_WOO_BOPO_BUNDLE_VERSION );
			wp_enqueue_script( 'woo-bopo-bundle-asual-address-js', VI_WOO_BOPO_BUNDLE_JS . 'address.min.js', array( 'jquery' ), VI_WOO_BOPO_BUNDLE_VERSION );
			wp_enqueue_script( 'woo-bopo-bundle-semantic-dropdown-js', VI_WOO_BOPO_BUNDLE_JS . 'dropdown.min.js', array( 'jquery' ), VI_WOO_BOPO_BUNDLE_VERSION );
			wp_enqueue_script( 'woo-bopo-bundle-semantic-transition-js', VI_WOO_BOPO_BUNDLE_JS . 'transition.min.js', array( 'jquery' ), VI_WOO_BOPO_BUNDLE_VERSION );
		}
	}
}